/**
 *  \file main.cpp
 *  Bugyuta demó az SDL-lel kombinált alakzat osztályok kipróbálására.
 *  Felrajzol pár alakzatot, melyek az egérmozgással együtt mozognak.
 *  (csak az irányt követik, nem a mozgás nagyságát)
 */
#include <iostream>
#include <fstream>
#include "memtrace.h"
#include "szakasz.h"
#include "kor.h"

/// segédfüggvény az egérmozgáshoz
/// @param x - relatív elmozdulás
/// @return -1, 0, vagy 1 az elmozdulás irányától függően
inline int delta(int x) {
#ifdef DEMO
     return 0;
#else
     return x == 0 ? 0 : x < 0 ? -1 : 1;
#endif // DEMO
}

/// segédfüggvény az SDL inicializálásához
/// @return suface pointer vagy 0
SDL_Surface* init() {
    if (SDL_Init( SDL_INIT_VIDEO ) < 0)
        return NULL;

    /// SDL jelen változata lezárja a szabványos kimeneteteket.
    /// Ezért újra kinyitjuk, hogy látszódjanak a konzolon a debug kiírások
    std::freopen( "CON", "w", stdout );
    std::freopen( "CON", "w", stderr );
    return SDL_SetVideoMode(1024, 768, 0, SDL_ANYFORMAT);
}

int main(int argc, char* argv[]) {
    SDL_Surface* screen = init();
    if (screen == NULL) {
        std::cerr << "Nem sikerult az SDL INIT: " << SDL_GetError() << std::endl;
        return 1;
    }
    Alakzat::setSurface(screen); // screen letárolása az "Alakzat-ba"

    const int N = 100;           // max 100 alakzatunk lehet
    Alakzat *idom[N];
    int db = 0;                  // most nincs egy se

    /// felrajzolunk egy szakaszt és egy kört
#ifndef DEMO
    idom[db++] = new Szakasz(Pont(20,40), 100, 0, WHITE);
    idom[db++] = new Kor(Pont(120,140), 50, BLUE);
#endif

    /// Eseménykezelõ ciklus.
    SDL_Event ev;
    while (SDL_WaitEvent(&ev) && ev.type != SDL_QUIT) {
        switch (ev.type) {
          case SDL_MOUSEMOTION: {           // egérmozgás esemény
            int dx = delta(ev.motion.xrel); // merre mozdult ?
            int dy = delta(ev.motion.yrel);
            for (int i = 0; i < db; i++) {  // végigmegyünk az összesen
                idom[i]->mozgat(Pont(dx, dy));  // kicsit mozdítunk rajta
                idom[i]->rajzol();          // kirajzoljuk
            }
            break;
          }

          case SDL_MOUSEBUTTONDOWN:         // egérgomb kezelése
            if (db < N) {
                if (ev.button.button == SDL_BUTTON_LEFT)
                    idom[db] = new Szakasz(Pont(ev.button.x, ev.button.y),
                                40, 80, GREEN);
                else
                    idom[db] = new Kor(Pont(ev.button.x, ev.button.y), 40, RED);
                idom[db]->rajzol();
                ++db;
            }
            break;
        }
    }
    for (int i = 0; i < db; i++) /// fel kell szabadítani minden objektumot
        delete idom[i];

    SDL_Quit();
    /// SDL mellékhatása, hogy közvetlenül meg kell hívni a memtrace ellenőrző
    /// funkcióját (exit ágon azért is lezárja a standard hibakimenetet).
    memtrace::mem_check();
    return 0;
}
